    '-------------------------------------------------------------------
    ' Locate in: ...\MediaMonkey\Scripts\Auto\Menu_iTunesReplayGain.vbs
    '
    ' Version: 2.6
    ' Date: 4 Jan 2017
    ' By Rick Parsons (partly based on UndoAnalyzeVolume by DiddeLeeDoo)
    ' Tested on Windows XP, 7 & 10 using MediaMonkey 4.1.14 and iTunes 12.5.4.
    '
    ' The Set ... ReplayGain options require "Analyze Volume" to have been run first.
    '     Preferably with the option "Player / Volume Leveling / When analyzing
    '     Track volume, also analyze Album volume" set
    '     For Album Volume to be set the whole album must be Analyzed together
    '
    ' It sets the iTunes SoundCheck marker to match the MM ReplayGain value
    '     either volume or track as you choose overwriting any value that may
    '     be there already (no backup).
    '
    ' It only works on MP3 files because these are the only
    '     ones that can have both ReplayGain and SoundCheck values.
    '
    ' NOTE: **** The Read & Clear SoundCheck options use Custom2 as a holding location.
    '    If this is used for anything else it will be corrupted. Change this to an
    '    unused custom field below.
    '
    ' Change log
    ' Note the version numbers for mp3rg2sc are separate and different
    '    v2.3 Initial version (number corresponds to SonosReplayGain)
    '    v2.5 Added function to extract iTunNORM and place it in a Custom variable
    '    v2.6 Refine & bug fix Read SoundCheck and Clear SoundCheck
    '-------------------------------------------------------------------

Sub OnStartup
    Set UI = SDB.UI
    ' Add a section to the main tools menu
    Set Mnu = UI.AddMenuItemSub(UI.Menu_Tools, 4, 2)
        Mnu.Caption = SDB.Localize("iTunes ReplayGain")
        Mnu.IconIndex = 39
        ' Set Album ReplayGain
    Set Mn1 = UI.AddMenuItem(Mnu, 0, 0)
        Mn1.Caption = "Set Album ReplayGain"
        Mn1.UseScript = Script.ScriptPath
        Mn1.OnClickFunc = "Album"
        Mn1.IconIndex = 39
        ' Set Track ReplayGain
    Set Mn1 = UI.AddMenuItem(Mnu, 0, 0)
        Mn1.Caption = "Set Track ReplayGain"
        Mn1.UseScript = Script.ScriptPath
        Mn1.OnClickFunc = "Track"
        Mn1.IconIndex = 39
        ' Clear ReplayGains
    Set Mn3 = UI.AddMenuItem(Mnu, 0, 0)
        Mn3.Caption = "Clear SoundCheck"
        Mn3.UseScript = Script.ScriptPath
        Mn3.OnClickFunc = "Clear"
        Mn3.IconIndex = 39
        ' Extract iTunNORM
    Set Mn3 = UI.AddMenuItem(Mnu, 0, 0)
        Mn3.Caption = "Read SoundCheck"
        Mn3.UseScript = Script.ScriptPath
        Mn3.OnClickFunc = "ReadSC"
        Mn3.IconIndex = 39
    ' Add a section to the Main window right click context menu
    Set Mnu = UI.AddMenuItemSub(UI.Menu_Pop_TrackList, 5, 6)
        Mnu.Caption = SDB.Localize("iTunes ReplayGain")
        Mnu.IconIndex = 39
        ' Set Album ReplayGain
    Set Mn1 = UI.AddMenuItem(Mnu, 0, 0)
        Mn1.Caption = "Set Album ReplayGain"
        Mn1.UseScript = Script.ScriptPath
        Mn1.OnClickFunc = "Album"
        Mn1.IconIndex = 39
        ' Set Track ReplayGain
    Set Mn1 = UI.AddMenuItem(Mnu, 0, 0)
        Mn1.Caption = "Set Track ReplayGain"
        Mn1.UseScript = Script.ScriptPath
        Mn1.OnClickFunc = "Track"
        Mn1.IconIndex = 39
        ' Clear SoundCheck
    Set Mn3 = UI.AddMenuItem(Mnu, 0, 0)
        Mn3.Caption = "Clear SoundCheck"
        Mn3.UseScript = Script.ScriptPath
        Mn3.OnClickFunc = "Clear"
        Mn3.IconIndex = 39
        ' Extract iTunNORM
    Set Mn3 = UI.AddMenuItem(Mnu, 0, 0)
        Mn3.Caption = "Read SoundCheck"
        Mn3.UseScript = Script.ScriptPath
        Mn3.OnClickFunc = "ReadSC"
        Mn3.IconIndex = 39
    End Sub

Sub Album(o)
    If SDB.SelectedSongList.Count > 0 Then
        Set dbT = SDB.SelectedSongList
        Set Prg = SDB.Progress
        j = 0
        Prg.MaxValue = dbT.Count - 1
        Prg.Text = SDB.Localize("Seting Album ReplayGain...")
        For i = 0 To dbT.Count - 1
            Dim sw
            sw="/a"
            Set Sng = dbT.Item(i)
            If Sng.LevelingAlbum = -999999 Then
                ' Set warning flag
                j = 1
                ' Use Track Volume instead
                sw = ""
                ' It is safe to carry on as mp3rg2sc doesn't do anything if there is no value there
            End If
            Prg.Value = i
            Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
	    ' Ignore it if the program doesn't exist
            If fso.FileExists(SDB.ScriptsPath & "Auto\mp3rg2sc.exe") Then
                Dim wsh : Set wsh = CreateObject("WScript.Shell")
		' It doesn't do any harm to call it for all files
                Call wsh.Run("""" & SDB.ScriptsPath & "Auto\mp3rg2sc.exe"" " & sw & " """ & Sng.Path & """", 0, True)
            End If
            If Prg.Terminate Then Exit For
        Next
        Set Prg = Nothing
        Set dbT = Nothing
        If j = 1 Then
            SDB.MessageBox SDB.Localize("Album Volume is not available on one or more tracks. Track Volume used"), mtError, Array(mbOk)
	End If
    Else
        SDB.MessageBox SDB.Localize("Please select the tracks to be processed"), mtError, Array(mbOk)
    End If
End Sub

Sub Track(o)
    If SDB.SelectedSongList.Count > 0 Then
        Set dbT = SDB.SelectedSongList
        Set Prg = SDB.Progress
        j = 0
        Prg.MaxValue = dbT.Count - 1
        Prg.Text=SDB.Localize("Setting Track ReplayGain...")
        For i = 0 To dbT.Count - 1
            Set Sng = dbT.Item(i)
            If Sng.Leveling = -999999 Then
                ' Set warning flag
                j = 1
                ' It is safe to carry on as mp3rg2sc doesn't do anything if there is no value there
            End If
            Prg.Value = i
            Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
	    ' Ignore it if the program doesn't exist
            If fso.FileExists(SDB.ScriptsPath & "Auto\mp3rg2sc.exe") Then
                Dim wsh : Set wsh = CreateObject("WScript.Shell")
		' It doesn't do any harm to call it for all files
                Call wsh.Run("""" & SDB.ScriptsPath & "Auto\mp3rg2sc.exe"" """ & Sng.Path & """", 0, True)
            End If
            If Prg.Terminate Then Exit For
        Next
        Set Prg = Nothing
        Set dbT = Nothing
        If j = 1 Then
            SDB.MessageBox SDB.Localize("Track Volume is not available on one or more tracks. Please re-analyse these tracks"), mtError, Array(mbOk)
        End If
    Else
        SDB.MessageBox SDB.Localize("Please select the tracks to be processed"), mtError, Array(mbOk)
    End If
End Sub

Sub Clear(o)
    If SDB.SelectedSongList.Count > 0 Then
        Set dbT = SDB.SelectedSongList
        Set Prg = SDB.Progress
        Prg.MaxValue = dbT.Count-1
        Prg.Text = SDB.Localize("Clearing SoundCheck...")
        For i = 0 To dbT.Count - 1
            Set Sng = dbT.Item(i)
            Prg.Value = i
            Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
	    ' Ignore it if the program doesn't exist
            If fso.FileExists(SDB.ScriptsPath & "Auto\mp3rg2sc.exe") Then
                Dim wsh : Set wsh = CreateObject("WScript.Shell")
		' It doesn't do any harm to call it for all files
                Call wsh.Run("""" & SDB.ScriptsPath & "Auto\mp3rg2sc.exe"" /r """ & Sng.Path & """", 0, True)
                ' Clear the save field to make it obvious
' **** Use an available Custom field in the line below. The default is Custom2
                Sng.Custom2 = ""
                Sng.UpdateDB
                Sng.WriteTags
            End If
            If Prg.Terminate Then Exit For
        Next
        Set Prg = Nothing
        Set dbT = Nothing
    Else
        SDB.MessageBox SDB.Localize("Please select the tracks to be processed"), mtError, Array(mbOk)
    End If
End Sub

Sub ReadSC(o)
    If SDB.SelectedSongList.Count > 0 Then
        Set dbT = SDB.SelectedSongList
        Set Prg = SDB.Progress
        Prg.MaxValue = dbT.Count - 1
        Prg.Text = SDB.Localize("Reading SoundCheck...")
        For i = 0 To dbT.Count - 1
            Set Sng = dbT.Item(i)
            Prg.Value = i
            Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
	    ' Ignore it if the program doesn't exist
            If fso.FileExists(SDB.ScriptsPath & "Auto\mp3rg2sc.exe") Then
' **** Use an available Custom field in the line below. The default is Custom2
                Sng.Custom2 = ShellRun("""" & SDB.ScriptsPath & "Auto\mp3rg2sc.exe"" /x """ & Sng.Path & """")
                If Sng.Custom2 <> "" Then
                    Sng.Custom2 = Sng.Custom2 & " = iTunNORM read on " & now()
                End If
                Sng.UpdateDB
                Sng.WriteTags
            End If
            If Prg.Terminate Then Exit For
        Next
        Set Prg = Nothing
        Set dbT = Nothing
    Else
        SDB.MessageBox SDB.Localize("Please select the tracks to be processed"), mtError, Array(mbOk)
    End If
End Sub

Function ShellRun(sCmd)

    'Run a shell command, returning the output as a string'

    Dim oShell
    Set oShell = CreateObject("WScript.Shell")

    'run command'
    Dim oExec
    Dim oOutput
    Set oExec = oShell.Exec(sCmd)
    Set oOutput = oExec.StdOut

    'handle the results as they are written to and read from the StdOut object'
    Dim s
    ' We only want the first line of STDOUT - either the result or an error message.
    If Not oOutput.AtEndOfStream Then
        s = oOutput.ReadLine
    End If

    ShellRun = s

End Function
